//+------------------------------------------------------------------+
//|                                                    Plot_News.mq4 |
//|                                                   David Louisson |
//|                                        http://www.metaquotes.net |
//+------------------------------------------------------------------+

//+------------------------------------------------------------------+
//|   Version 1.0    05 Feb 2007                                     |
//|                                                                  |
//|   Reads news from NEWS.CSV file and plots appropriate            |
//|   objects on chart                                               |
//+------------------------------------------------------------------+
//
//+------------------------------------------------------------------+
//|   Amendments (Greg Spinner)                                      |                                                  
//|      v1.07.1 - 06/12/2007 - 07/28/2007                           |            
//|      1) Allow filtering by external variable                     |
//|      2) Read file into buffer prior to starting rep selection and|
//|         object creation                                          |
//|      3) Allow filter by up to 6 reports for a given time         |
//|         i.e. past reports will only be displayed if their text   | 
//|         matches that of a report announced at the given time     |  
//|      4) Allow filter by the next due report(s).                  |
//|      5) Refresh news plot when a new bar is created              | 
//|      6) Display pre-news pip movement                            |
//|                                                                  |
//|      v1.07.2 - 06/12/2007 - 07/28/2007                           |             
//|      7) Incorporate NewsCaptureCSV logic                         |
//|                                                                  |
//|      v1.07.3 - 06/12/2007 - 07/31/2007                           |            
//|      8) Changes NewsCapture logic as follows:-                   | 
//|          i) OHLC values output based on timeframe of chart;      |
//|         ii) each bar's set of values is output as a row;         |
//|      9) New parameter - useDateFilter, which works in            |                                                         
//|         conjunction with dateFilter (i.e dateFilter              |                           
//|         only active if this parameter set to TRUE)               |
//|     10) New icon showing pip movement in the time period         |                                                     
//|         set by new paramaters preNewsStart and preNewsEnd.       |                                             
//|         Time span displayed takes into account new parameter     |                                                   
//|         chartAheadGMT. Calulation is (closing price of bar       |                                                  
//|         preNewsEnd bars ago) minus (open price of bar            |                                           
//|         preNewsStart bars ago).                                  |                      
//|     11) New parameter markOnly, used in conjunction with         |                                                    
//|         filters that marks the reports that would have been      |                                                  
//|         left after application of the filter but leaves all      |                                                  
//|         reports on the chart.                                    |                    
//|                                                                  |
//+------------------------------------------------------------------+
//
#property copyright "David Louisson"
#property link      "http://www.metaquotes.net"

#include <stderror.mqh>
#include <stdlib.mqh>

#property indicator_chart_window

// NEWS.CSV file must exist in: \metatrader 4\experts\files
extern   string   FileName          = "news.csv";   // News file name
extern   int      VertSpacing       = 5;            // # pips spacing between symbols plotted against same bar
extern   int      SymbolSize        = 2;            // size of symbols being plotted on the chart

// Filters (set parameter to FALSE to suppress plot):
extern   datetime FromDate          = D'1990.01.01 00:00';
extern   datetime ThruDate          = D'2009.12.31 23:59';
//* v1.07.1/2/3 - start -------------------------------------------------------------------------------------------------*
extern   int      chartAheadGMT     = 1;  // how many hours the chart is ahead of GMT  
extern   string   textFilter        = "";           // only include reports containing this string
extern   bool     useDateFilter     = false; // works in conjunction with dateFilter
extern   datetime dateFilter        = D'1990.01.01 00:00';   // only plot reports with the same description as those at this time
extern   bool     filterByNext      = false; // only plot reports with the same description(s) as the next report(s)
extern   bool     markOnly          = false; // an arrow is drawn to indicate reports remaining after filtering but all news is plotted 
extern   int      preNewsStart      = 4;   // bars before news when pre News move starts
extern   int      preNewsEnd        = 2;   // bars before news when pre News move ends
extern   bool     buildNewsCSV      = false;        // if true, build CSV file of highs and lows +- 60 minutes around each news release 
extern   int      preReleaseBars    = 10;
extern   int      postReleaseBars   = 10;  
extern   string   seperatorChar     = ","; 
extern   string   suffix            = "csv";
//* v1.07.1/2/3 - end-- -------------------------------------------------------------------------------------------------*
extern   bool     Plot_High         = TRUE;         // High impact   = 3 (red plot)
extern   bool     Plot_Medium       = TRUE;         // Medium impact = 2 (orange plot)
extern   bool     Plot_Low          = TRUE;         // Low impact    = 1 (yellow plot)
extern   bool     Plot_USD          = TRUE;         // USD = 1 (wingding character #140)
extern   bool     Plot_CAD          = TRUE;         // CAD = 2 (wingding character #141)
extern   bool     Plot_EUR          = TRUE;         // EUR = 3 (wingding character #142)
extern   bool     Plot_GBP          = TRUE;         // GBP = 4 (wingding character #143)
extern   bool     Plot_CHF          = TRUE;         // CHF = 5 (wingding character #144)
extern   bool     Plot_JPY          = TRUE;         // JPY = 6 (wingding character #145)
extern   bool     Plot_AUD          = TRUE;         // AUD = 7 (wingding character #146)
extern   bool     Plot_NZD          = TRUE;         // NZD = 8 (wingding character #147)

int      handle;

int      eCount;              // Event counter
datetime eTime[9999];         // Event date/time, format = yyyy.mm.dd<space>hh:mm
int      eNumber[9999];       // Event seq# (same date/time use ascending seq#s)
int      eWingding[9999];     // Wingding code for implicated currency, as above
int      eImpact[9999];       // 1=low impact; 2=medium impact; 3=high impact
string   eCurrency[9999];     // Currency/ies on whose charts object will display; ALL = all currencies 
string   ePeriods[9999];      // Time period settings on which object will display (M1,M5,M15,M30,H1,H4,D1,W1,MTH; 0=no, 1=yes)
string   eText1[9999];        // First descr line, format = hh:mm XXX Y "event description"
                              //   where XXX=currency; Y=H/M/L impact
string   eText2[9999];        // Second descr line, shows Actual, Forecast, Previous values

//* v1.07.1/2/3 - start -------------------------------------------------------------------------------------------------*
bool filterByDate = false;    // date filtering active
string requiredDate;          // date to use as basis for filtering
string repText [6];           // descriptions of reports happening at selected time
string repCurrency [6];       // currency of each report in repText
int textPos = 0;              // how many report decsription on selected bar
datetime latestBar;           // hold the time of the latest bar
datetime preNewsStartTime;
datetime preNewsEndTime;
int preNewsStartBar;
int preNewsEndBar;
int preNewsPips;
string   newsCaptureRec;
int      NC;
//* v1.07.1/2/3 - end-- -------------------------------------------------------------------------------------------------*

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{
   plotNews();
}

//+------------------------------------------------------------------+
//| Extract quoted string from supplied text                         |
//+------------------------------------------------------------------+
string extractText(string eText)
{
   string quote = "\"";
   int quote1Pos = StringFind(eText,quote,0);
   int quote2Pos = StringFind(eText,quote,quote1Pos + 1);
   return (StringSubstr(eText,quote1Pos + 1,quote2Pos - quote1Pos - 1));
}

//+------------------------------------------------------------------+
//| Plot news events                                                 |
//+------------------------------------------------------------------+
void plotNews()
{

//---- indicators

   string objname, sym1, sym2, cmp1, cmp2;
   int    barno, dir1;
// v1.07.1/2/3 - start -------------------------------------------------------------------------------------------*
   double vertpos, vertpos1, vertpos2;
// v1.07.1/2/3 - end ---------------------------------------------------------------------------------------------*

   handle = FileOpen(FileName, FILE_CSV|FILE_READ,';');          // Open NEWS.CSV
   if(handle==0)
      Comment("File "+FileName+" not found.");

//* v1.07.1/2/3 - start -------------------------------------------------------------------------------------------------*
   if (buildNewsCSV)
   {           
      int n;
      string filename="NewsDataCapture"+Symbol()+"."+suffix;
      NC=FileOpen(filename, FILE_CSV|FILE_WRITE, '|');
      if (NC == -1)
      {
         int err=GetLastError();
         Print("CSV File open error (",err,"): ",ErrorDescription(err));
         return(0);
      }
      newsCaptureRec="Rep No"+seperatorChar+"Rep Time"+seperatorChar+"Impact"+seperatorChar+"Currency"+seperatorChar+"Text1"+seperatorChar+"Text2"+seperatorChar+"Time"+seperatorChar+"Open"+seperatorChar+"High"+seperatorChar+"Low"+seperatorChar+"Close"+seperatorChar;
   
      FileWrite(NC,newsCaptureRec);
   }
//* v1.07.1/2/3 - end ---------------------------------------------------------------------------------------------------*
      
   if (useDateFilter && dateFilter != D'1990.01.01 00:00') 
   {
      filterByDate = true;
      requiredDate = TimeToStr(dateFilter,TIME_DATE|TIME_MINUTES);  
   }

// Read full file into buffers before starting selection process
   for(eCount = 0; !FileIsEnding(handle); eCount++)              // Read from NEWS.CSV; one line per event
   {
      eTime[eCount]     = StrToTime(FileReadString(handle));     // Event date/time
      eNumber[eCount]   = StrToInteger(FileReadString(handle));  // Event seq# (same date/time have ascending seq#s)
      eWingding[eCount] = StrToInteger(FileReadString(handle));  // Wingding code denoting implicated currency
      eImpact[eCount]   = StrToInteger(FileReadString(handle));  // Impact: 1=low; 2=medium; 3=High
      eCurrency[eCount] = FileReadString(handle);                // Currency/ies on whose charts object will display; ALL = all currencies
      ePeriods[eCount]  = FileReadString(handle);                // Time period settings on which object will display (M1,M5,M15,M30,H1,H4,D1,W1,MTH; 0=no, 1=yes)
      eText1[eCount]    = FileReadString(handle);                // First line of description
      eText2[eCount]    = FileReadString(handle);                // Second line of description

      if (filterByNext)
      {
         if (eTime[eCount] >= Time[0] && eTime[eCount] >= TimeCurrent() && (eCurrency[eCount] == StringSubstr(Symbol(),0,3) || eCurrency[eCount] == StringSubstr(Symbol(),3,3)))
         {
            filterByDate = true;
            requiredDate = TimeToStr(eTime[eCount],TIME_DATE|TIME_MINUTES);
            Print("required date = ",requiredDate);
            filterByNext = false;
         }
      }
      if (filterByDate && TimeToStr(eTime[eCount],TIME_DATE|TIME_MINUTES) == requiredDate)
      {
         if (textPos < 6)
         {
            repText[textPos] = extractText(eText1[eCount]);
            repCurrency[textPos] = eCurrency[eCount];
            textPos++;
         }
      }
// v1.07.1/2/3 - start -------------------------------------------------------------------------------------------*
   }
   for (int j=0;j<eCount;j++)
   {
// v1.07.1/2/3 - end   -------------------------------------------------------------------------------------------*
      if (eNumber[j] == 0 )      continue;                  // Invalid entry, or EOF
//    if (eWingding[j] == 0 )    continue;
//    if (eImpact[j] == 0 )      continue;

      if (eTime[j] < FromDate) continue;                    // Skip if outside entered date range
      if (eTime[j] > ThruDate) continue;
      
      if (eImpact[j] == 1 && Plot_Low == FALSE)    continue;                     // Skip if impact outside entered values
      if (eImpact[j] == 2 && Plot_Medium == FALSE) continue;
      if (eImpact[j] == 3 && Plot_High == FALSE)   continue;
      
      if (eWingding[j] == 140 && Plot_USD == FALSE)   continue;                  // Skip if currency outside entered values
      if (eWingding[j] == 141 && Plot_CAD == FALSE)   continue;
      if (eWingding[j] == 142 && Plot_EUR == FALSE)   continue;
      if (eWingding[j] == 143 && Plot_GBP == FALSE)   continue;
      if (eWingding[j] == 144 && Plot_CHF == FALSE)   continue;
      if (eWingding[j] == 145 && Plot_JPY == FALSE)   continue;
      if (eWingding[j] == 146 && Plot_AUD == FALSE)   continue;
      if (eWingding[j] == 147 && Plot_NZD == FALSE)   continue;

      if ( StringSubstr(ePeriods[j],0,1) == "0" && Period() == PERIOD_M1  )   continue;    // Skip if not to be shown on this period's chart
      if ( StringSubstr(ePeriods[j],1,1) == "0" && Period() == PERIOD_M5  )   continue;
      if ( StringSubstr(ePeriods[j],2,1) == "0" && Period() == PERIOD_M15 )   continue;
      if ( StringSubstr(ePeriods[j],3,1) == "0" && Period() == PERIOD_M30 )   continue;
      if ( StringSubstr(ePeriods[j],4,1) == "0" && Period() == PERIOD_H1  )   continue;
      if ( StringSubstr(ePeriods[j],5,1) == "0" && Period() == PERIOD_H4  )   continue;
      if ( StringSubstr(ePeriods[j],6,1) == "0" && Period() == PERIOD_D1  )   continue;
      if ( StringSubstr(ePeriods[j],7,1) == "0" && Period() == PERIOD_W1  )   continue;
      if ( StringSubstr(ePeriods[j],8,1) == "0" && Period() == PERIOD_MN1 )   continue;

      bool skipflag = FALSE;                                                          // Skip if currency outside entered values
      if (eCurrency[j] != "ALL")
        {
          skipflag = TRUE;
          sym1 = StringSubstr(Symbol(),0,3);
          sym2 = StringSubstr(Symbol(),3,3);
          cmp1 = StringSubstr(eCurrency[j],0,3);
          if (StringLen(eCurrency[j]) > 3) 
          {
            cmp2 = StringSubstr(eCurrency[j],3,3);
            if ((cmp1 == "ALL" || cmp1 == sym1 || cmp1 == sym2) && (cmp2 == "ALL" || cmp2 == sym1 || cmp2 == sym2)) skipflag = FALSE;   
          }
          else
          {
            if (cmp1 == "ALL" || cmp1 == sym1 || cmp1 == sym2) skipflag = FALSE;   
          }  
        }
      if (skipflag == TRUE) continue;  

//* v1.07.1/2/3 - start -------------------------------------------------------------------------------------------------*
      if (textFilter != "")
      {
         if (StringFind(eText1[j],textFilter,0) == -1)
         {
            if (markOnly == false) continue;
         }
         else
         {
            barno = iBarShift(NULL,0,eTime[j],FALSE);
            vertpos = High[barno] + 20*Point; 
            vertpos1 = High[barno] + 25*Point; 
            vertpos2 = High[barno] + 30*Point; 
            ObjectCreate(eText1[j]+"_arrow",OBJ_ARROW,0,eTime[j],vertpos);
            ObjectSet(eText1[j]+"_arrow",OBJPROP_ARROWCODE,234);
            ObjectSet(eText1[j]+"_arrow",OBJPROP_COLOR,Navy);
            
            ObjectCreate(eText1[j]+"_arrow1",OBJ_ARROW,0,eTime[j],vertpos1);
            ObjectSet(eText1[j]+"_arrow1",OBJPROP_ARROWCODE,234);
            ObjectSet(eText1[j]+"_arrow1",OBJPROP_COLOR,MediumBlue);
               
            ObjectCreate(eText1[j]+"_arrow2",OBJ_ARROW,0,eTime[j],vertpos2);
            ObjectSet(eText1[j]+"_arrow2",OBJPROP_ARROWCODE,234);
            ObjectSet(eText1[j]+"_arrow2",OBJPROP_COLOR,Blue);
         }
      }
      
      if (filterByDate)
      {
         skipflag = true;
         for (int z=0;z<textPos;z++)
         {
            if (extractText(eText1[j]) == repText[z] && eCurrency[j] == repCurrency[z])
            {
               barno = iBarShift(NULL,0,eTime[j],FALSE);
               vertpos = High[barno] + 20*Point; 
               vertpos1 = High[barno] + 25*Point; 
               vertpos2 = High[barno] + 30*Point; 
               ObjectCreate(eText1[j]+"_arrow",OBJ_ARROW,0,eTime[j],vertpos);
               ObjectSet(eText1[j]+"_arrow",OBJPROP_ARROWCODE,234);
               ObjectSet(eText1[j]+"_arrow",OBJPROP_COLOR,Navy);
               
               ObjectCreate(eText1[j]+"_arrow1",OBJ_ARROW,0,eTime[j],vertpos1);
               ObjectSet(eText1[j]+"_arrow1",OBJPROP_ARROWCODE,234);
               ObjectSet(eText1[j]+"_arrow1",OBJPROP_COLOR,MediumBlue);
               
               ObjectCreate(eText1[j]+"_arrow2",OBJ_ARROW,0,eTime[j],vertpos2);
               ObjectSet(eText1[j]+"_arrow2",OBJPROP_ARROWCODE,234);
               ObjectSet(eText1[j]+"_arrow2",OBJPROP_COLOR,Blue);
               skipflag = false;
               break;
            }   
         }
      }
      if (skipflag == true && markOnly == false) continue;
      
      preNewsStartBar = iBarShift(NULL,0,eTime[j]) + preNewsStart;
      preNewsEndBar = iBarShift(NULL,0,eTime[j]) + preNewsEnd;
      preNewsPips = MathRound((iClose(NULL,0,preNewsEndBar) - iOpen(NULL,0,preNewsStartBar))/Point);
      preNewsStartTime = Time[preNewsStartBar];
      preNewsEndTime = Time[preNewsEndBar - 1];
      //Print("eTime[j] = ",TimeToStr(eTime[j]),", preNewsS = ",TimeToStr(preNewsStartTime),", preNewsE = ",TimeToStr(preNewsEndTime));
               
      barno = iBarShift(NULL,0,eTime[j],FALSE);
      string preNewsText = "Movement between "+TimeToStr(preNewsStartTime-(chartAheadGMT * 3600))+" and "+TimeToStr(preNewsEndTime-(chartAheadGMT*3600))+" = "+preNewsPips;
      vertpos = High[barno] + 10*Point; 
      ObjectCreate(TimeToStr(eTime[j])+"_preNews",OBJ_ARROW,0,eTime[j],vertpos);
      ObjectSet(TimeToStr(eTime[j])+"_preNews",OBJPROP_ARROWCODE,171);
      ObjectSet(TimeToStr(eTime[j])+"_preNews",OBJPROP_COLOR,LightSalmon);
      ObjectSetText(TimeToStr(eTime[j])+"_preNews",preNewsText);                                        
      
//* v1.07.1/2/3 - end   -------------------------------------------------------------------------------------------------*
           
      objname = eText1[j];                                                       // Set object name = first descr line
      barno = iBarShift(NULL,0,eTime[j],FALSE);                                  // Number of bars back from current
//    if (eWingding[j] == 0) barno++;

      dir1 = 1;
      if (eNumber[j] < 0) dir1 = -1;
      
      if (VertSpacing * eNumber[j] > 0) 
        vertpos = iLow(NULL,0,barno) - VertSpacing * eNumber[j] * Point;         // Plot object VertSpacing pips x seq# below low of relevant bar
      else
        vertpos = iHigh(NULL,0,barno) - VertSpacing * (eNumber[j]+dir1) * Point;    // Plot object VertSpacing pips x seq# above high of relevant bar
      if (eWingding[j] == 0)
       {  
        ObjectCreate(objname,OBJ_TEXT,0,eTime[j],vertpos);                
        ObjectSet(objname,OBJPROP_ANGLE,0);
        ObjectSetText(objname,eText2[j],9,"Verdana",White);                      // Set object descr = second descr line
       } 
      else
       {
        ObjectCreate(objname,OBJ_ARROW,0,eTime[j],vertpos);                
        ObjectSet(objname,OBJPROP_ARROWCODE,eWingding[j]);                       // Set wingding code
        ObjectSet(objname,OBJPROP_WIDTH,SymbolSize);                                  // Set size of object
        ObjectSetText(objname,eText2[j]);                                        // Set object descr = second descr line
       } 
      ObjectSet(objname,OBJPROP_COLOR,White);                                         // Default color
      if (eImpact[j] == 1) ObjectSet(objname,OBJPROP_COLOR,Yellow);              // Low impact = yellow
      if (eImpact[j] == 2) ObjectSet(objname,OBJPROP_COLOR,Orange);              // Medium impact = orange
      if (eImpact[j] == 3) ObjectSet(objname,OBJPROP_COLOR,Red);                 // High impact = red
      if (eImpact[j] == 4) ObjectSet(objname,OBJPROP_COLOR,LimeGreen);           // Allow other primary colors
      if (eImpact[j] == 5) ObjectSet(objname,OBJPROP_COLOR,DodgerBlue);        
      if (eImpact[j] == 6) ObjectSet(objname,OBJPROP_COLOR,Magenta);        
      if (eImpact[j] == 7) ObjectSet(objname,OBJPROP_COLOR,Aqua);        
      if (eImpact[j] == 8) ObjectSet(objname,OBJPROP_COLOR,Goldenrod);        
      if (eImpact[j] == 9) ObjectSet(objname,OBJPROP_COLOR,Plum);        
      if (eImpact[j] == 10) ObjectSet(objname,OBJPROP_COLOR,Silver);     
         
//* v1.07.1/2/3 - start -------------------------------------------------------------------------------------------------*
      if (buildNewsCSV && eTime[j] < TimeCurrent()) newsCaptureCSV(j);
//* v1.07.1/2/3 - end ---------------------------------------------------------------------------------------------------*
   }
//* v1.07.1/2/3 - start -------------------------------------------------------------------------------------------------*
   if (NC > 0) FileClose(NC);
//* v1.07.1/2/3 - end ---------------------------------------------------------------------------------------------------*
 
   int debug, i;
   debug=FileOpen("debug.txt", FILE_CSV|FILE_WRITE, '|');
   for( i=0; i<eCount; i++)
     FileWrite(debug,i,TimeToStr(eTime[i]),eNumber[i],eWingding[i],eImpact[i],eCurrency[i],eText1[i],eText2[i]);
   FileClose(debug);

   FileClose(handle);

//----
   return(0);
}

//* v1.07.1/2/3 - start -------------------------------------------------------------------------------------------------*
//+------------------------------------------------------------------+
//| Build CSV of prices                                              |
//+------------------------------------------------------------------+

void newsCaptureCSV (int n)
{
  
//   for(n=0; n<eCount-1; n++)
//   {
      int      reportBar    = iBarShift(NULL,0,eTime[n]);
      int startBar = reportBar + preReleaseBars;
      int endBar = MathMax(reportBar - postReleaseBars,0);
      if (startBar > iBars(NULL,0)) return(0);
      
      string newsCaptureBase = n+seperatorChar+TimeToStr(eTime[n])+seperatorChar+eImpact[n]+seperatorChar+eCurrency[n]+seperatorChar+eText1[n]+seperatorChar+eText2[n];

      for (int z=startBar;z>=endBar;z--)
      {
         newsCaptureRec = newsCaptureBase+seperatorChar+TimeToStr(iTime(NULL,0,z))+seperatorChar+iOpen(NULL,0,z)+seperatorChar+iHigh(NULL,0,z)+seperatorChar+iLow(NULL,0,z)+seperatorChar+iClose(NULL,0,z);            
         FileWrite(NC,newsCaptureRec);
      }
      
   //}
}
//* v1.07.1/2/3 - end ---------------------------------------------------------------------------------------------------*

//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
{
//----
   int i;                                  // Delete created objects when indicator is removed
   for(i=0;i<eCount;i++)
   {
      if(eText1[i] != "")
      {
         ObjectDelete(eText1[i]);
// v1.07.1/2/3 - start -------------------------------------------------------------------------------------------*
         ObjectDelete(eText1[i]+"_arrow");
         ObjectDelete(eText1[i]+"_arrow1");
         ObjectDelete(eText1[i]+"_arrow2");
         ObjectDelete(TimeToStr(eTime[i])+"_preNews");
// v1.07.1/2/3 - end   -------------------------------------------------------------------------------------------*
      }
   }
     
//----
   return(0);
}

//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
{
//----
//* v1.07.1/2/3 - start -------------------------------------------------------------------------------------------------*
   if (Time[0] != latestBar)
   {
      //Print("Time[0] = ",TimeToStr(Time[0],TIME_DATE|TIME_MINUTES),", latest bar = ",TimeToStr(latestBar,TIME_DATE|TIME_MINUTES));
      //Print("------------");
      latestBar = Time[0];
      deinit();
      plotNews();
   }
      
//* v1.07.1/2/3 - end-- -------------------------------------------------------------------------------------------------*

  
//----
   return(0);
}


